#include <stdio.h>
#include <windows.h>

// Define how many items the circular buffer can hold.
#define SIZE 11

// Here is the data structure that defines a circular type buffer.
// The array has SIZE+2 entries so that it can hold SIZE pieces of
// data, an extra space that is needed so that there is someplace to
// point the front and back indexes at when the circular buffer is full,
// and a null charater ('\0') to denote the end of a string (the buffer
// will be printed out as one whole string). In this circular buffer,
// the front index points to the array entry that is just ahead of the
// most recently entered data item and the back index points to the array
// entry just before the data item that will be removed next. Notice that
// neither front nor back point directly to a data item in the circular
// buffer, they always point to empty array entires.
typedef struct
{
    char buffer[SIZE+2];
    unsigned int front;  // index of next empty spot
    unsigned int back;   // index just before the first filled spot
} Buffer;


// Prototype for the printBuffer function (defined below).
void printBuffer(Buffer* buffer_p, int bufferNo);


// Function that puts an item into the circular buffer data structure.
void putInBuffer(char item, Buffer* buffer_p, int bufferNo)
{
   // You need to write the code that puts an item into the
   // next available entry in the buffer and then adjusts the
   // front index. Be sure to "wrap" the front index when it
   // gets to the end of the buffer part of the array (remember
   // that the last entry in the array holds a null character
   // so that the array is in fact a null terminated string).
   // Notice that the parameter buffer_p is a pointer to the
   // Buffer data structure that holds the array to be manipulated.
   // buffer_p is NOT a pointer to the array of Buffers defined
   // in the program. Also, you do not need to make any use of
   // the parameter bufferNo. The bufferNo variable is only needed
   // for the call to printBuffer (so that printBuffer knows what
   // column to put the output in).

   // Print the circular buffer.
   printBuffer(buffer_p, bufferNo);

}//putInBuffer


// Function that removes an item from the circular buffer data structure.
char takeFromBuffer(Buffer* buffer_p, int bufferNo)
{
   char result = 0;

   // You need to write the code that removes the first available
   // item from the buffer and then adjusts the back index. Be sure
   // to "wrap" the back index when it gets to the end of the data
   // part of the array. Also, put the character '0' into the array
   // entry from which the data item was just removed, so that the
   // buffer looks better when it is printed out.

   // Print the circular buffer.
   printBuffer(buffer_p, bufferNo);

   return result;
}//takeFromBuffer


// Function that prints a circular buffer in its own column in the output.
void printBuffer(Buffer* buffer_p, int bufferNo)
{
   int index1, index2;
   char firstChar, secondChar;
   // a string of 70 spaces
   char spaces[] = "                                                                      ";

   // print the circular buffer and show both indexes
   if (buffer_p->front >= buffer_p->back)
   {  index1 = buffer_p->back;
      index2 = buffer_p->front;
      firstChar = '|';
      secondChar = '^';
      if (index1 == index2)
      {
         firstChar = '*';
         secondChar = ' ';
         index2++; //so that index2-index1-1 is 0
      }
   }
   else if (buffer_p->front < buffer_p->back)
   {  index1 = buffer_p->front;
      index2 = buffer_p->back;
      firstChar = '^';
      secondChar = '|';
   }
   printf("%.*s%s\n%.*s%.*s%c%.*s%c\n", bufferNo*(SIZE+2), spaces, buffer_p->buffer,
                                        bufferNo*(SIZE+2), spaces, index1, spaces, firstChar,
                                        index2-index1-1, spaces, secondChar);
}//printBuffer


// Function that initialzes all of the buffers.
void initializeBuffers(Buffer* buffers, int bufferCount)
{
   int i, j;

   // initialize the circular buffers to be empty
   for (i = 0; i < bufferCount; i++)
      for (j = 0; j < SIZE+1; j++)
        (buffers[i].buffer)[j] = '0';
   // null terminate each  of the buffers
   for (i = 0; i < bufferCount; i++)
      (buffers[i].buffer)[SIZE+1] = '\0';

   // initialize the circular buffer indexes
   for (i = 0; i < bufferCount; i++)
   {  buffers[i].front = 0;
      buffers[i].back = SIZE;
   }

   printf("Here is the initial state of the buffers.\n\n");
   // print column headings (each column is SIZE+2 characters wide)
   for (i = 0; i < bufferCount; i++)
      printf("  buffer[%d]%.*s", i, (SIZE+2)-11, "          ");
   printf("\n\n");
   for (i = 0; i < bufferCount; i++)  // print each buffer
      printBuffer(&(buffers[i]), i);

}//initializeBuffers


/*
    Use this function to print out meaningful error messages.
*/
void printError(char* functionName)
{   LPVOID lpMsgBuf;
    int error_no;
    error_no = GetLastError();
    FormatMessage(
         FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
         NULL,
         error_no,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
         (LPTSTR) &lpMsgBuf,
         0,
         NULL
    );
    // Display the string.
    fprintf(stderr, "\n%s failed on error %d: ", functionName, error_no);
    fprintf(stderr, lpMsgBuf);
    // Free the buffer.
    LocalFree( lpMsgBuf );
    ExitProcess(1);  // terminate the program
}//printError
